pub mod render;

pub trait RenderInterface {
    type Surface: render::Surface;
    fn render_target_mut(&mut self) -> &mut Self::Surface;

    fn submit(&mut self);
}

pub mod event;

pub trait EventInterface {
    fn poll_event(&mut self) -> Option<event::WindowEvent>;
    fn wait_for_event(&mut self);
}

pub mod text;

pub trait TextInterface {
    type Surface: render::Surface;

    fn render_text<'l>(&'l self, buffer: &text::TextBuffer, sz: f32) -> Self::Surface;
}

pub trait PlatformSpec {
    type Surface: render::Surface;
    type Render: RenderInterface<Surface = Self::Surface>;
    type Event: EventInterface;
    type Text: TextInterface<Surface = Self::Surface>;

    fn render(&self) -> &Self::Render;
    fn render_mut(&mut self) -> &mut Self::Render;

    fn event(&self) -> &Self::Event;
    fn event_mut(&mut self) -> &mut Self::Event;

    fn text(&self) -> &Self::Text;
    fn text_mut(&mut self) -> &mut Self::Text;

    // subset access
    fn render_text_mut(&mut self) -> (&mut Self::Render, &mut Self::Text);
}

pub type PlatformPainter<'p, P> =
    <<<P as PlatformSpec>::Render as RenderInterface>::Surface as render::Surface>::Painter<'p>;

impl<
        'l,
        RI: 'l + RenderInterface,
        EI: 'l + EventInterface,
        TI: 'l + TextInterface<Surface = RI::Surface>,
    > PlatformSpec for (RI, EI, TI)
{
    type Surface = RI::Surface;
    type Render = RI;
    type Event = EI;
    type Text = TI;

    fn render(&self) -> &Self::Render {
        &self.0
    }
    fn render_mut(&mut self) -> &mut Self::Render {
        &mut self.0
    }

    fn event(&self) -> &Self::Event {
        &self.1
    }
    fn event_mut(&mut self) -> &mut Self::Event {
        &mut self.1
    }

    fn text(&self) -> &Self::Text {
        &self.2
    }
    fn text_mut(&mut self) -> &mut Self::Text {
        &mut self.2
    }

    fn render_text_mut(&mut self) -> (&mut Self::Render, &mut Self::Text) {
        (&mut self.0, &mut self.2)
    }
}
