use crate::geom::{IPoint, IRect, IVector};

/// Simple colour representation
pub struct Colour {
    comp: [u8; 4],
}

/// Type alias for Americans
pub type Color = Colour;

impl Colour {
    pub fn new_rgb(r: u8, g: u8, b: u8) -> Self {
        Self {
            comp: [r, g, b, 255u8],
        }
    }
    pub fn new_rgba(r: u8, g: u8, b: u8, a: u8) -> Self {
        Self { comp: [r, g, b, a] }
    }

    /// Red component of colour
    pub fn r(&self) -> u8 {
        self.comp[0]
    }
    /// Green component of colour
    pub fn g(&self) -> u8 {
        self.comp[1]
    }
    /// Blue component of colour
    pub fn b(&self) -> u8 {
        self.comp[2]
    }
    /// Alpha component of colour (0=transparent,255=opaque)
    pub fn a(&self) -> u8 {
        self.comp[3]
    }

    pub fn to_rgba(&self) -> u32 {
        u32::from_le_bytes(self.comp)
    }
}

pub struct ColourMask {
    pub base: u8,
    pub len: u8,
}

#[derive(Clone, Copy, Debug)]
pub enum AccessOrder {
    RowMajor,
    ColumnMajor,
}

pub trait SurfaceFormat {
    const PIXEL_STRIDE: usize;
    const ACCESS: AccessOrder;
    const RED_MASK: ColourMask;
    const GREEN_MASK: ColourMask;
    const BLUE_MASK: ColourMask;
    const ALPHA_MASK: ColourMask;
}

pub trait Surface {
    fn size(&self) -> IVector;

    type Painter<'l>: Painter<Surface = Self> + 'l
    where
        Self: 'l;
    fn painter(&mut self) -> Option<Self::Painter<'_>>;
}

pub trait Painter {
    type Surface: Surface;

    fn blit(&mut self, source: &Self::Surface, source_area: IRect, target: IPoint);
    fn fill(&mut self, area: IRect, col: Colour);
    fn outline_rect(&mut self, rect: IRect, col: Colour);
}
