use crate::{
    component::Component,
    input::InputState,
    layout::{LayoutCache, LayoutNode, LayoutNodeAccess},
    platform::{
        render::{Painter, Surface},
        PlatformPainter, PlatformSpec, RenderInterface, TextInterface,
    },
};

mod button;
mod frame;
mod group;
mod label;

pub use frame::Frame;
pub use group::PlainGroup;
pub use label::Label;

pub trait Widget<P: PlatformSpec, C: Component> {
    fn poll(&mut self, input_state: Option<&InputState>) -> Vec<C::Msg>;

    fn layout_node(&self) -> LayoutNodeAccess;
    fn layout_node_mut(&mut self) -> &mut LayoutNode;
    fn render(&self, painter: &mut PlatformPainter<'_, P>, ti: &P::Text);
}

pub struct BuildContext {
    pub(crate) lcache: std::rc::Rc<LayoutCache>,
}
