use crate::{
    geom::{IPoint, IRect}, layout::{LayoutNode, LayoutNodeAccess, LeafLayoutNode}, platform::{
        render::{Colour, Painter, Surface}, text::TextBuffer, PlatformPainter, PlatformSpec, TextInterface
    }, Component, Widget
};

use super::BuildContext;

pub struct Label<P: PlatformSpec, C: Component> {
    lnode: LeafLayoutNode,
    buffer: TextBuffer,
    _ghost: std::marker::PhantomData<(P, C)>,
}

impl<P: PlatformSpec, C: Component> Label<P, C> {
    pub fn new(bc: &BuildContext) -> Self {
        Self {
            lnode: LayoutNode::new(bc.lcache.clone()).into(),
            buffer: TextBuffer::new("example text here"),
            _ghost: Default::default(),
        }
    }
}

impl<P: PlatformSpec, C: Component> Widget<P, C> for Label<P, C> {
    fn poll(
        &mut self,
        input_state: Option<&crate::input::InputState>,
    ) -> Vec<<C as Component>::Msg> {
        vec![]
    }

    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(&self.lnode)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        &mut self.lnode
    }

    fn render(&self, painter: &mut PlatformPainter<P>, ti: &P::Text) {
        painter.fill(
            self.layout_node().cached_rect().unwrap(),
            Colour::new_rgb(64, 64, 64),
        );
        let text_surf = ti.render_text(&self.buffer, 0.0);
        painter.blit(&text_surf, IRect::new_from_size(IPoint::ORIGIN, text_surf.size()), self.layout_node().cached_rect().unwrap().tl());
    }
}
